#ifndef RPM_INTERNAL_H
#define RPM_INTERNAL_H

#include <tcl.h>
#include <rpm.h>

#ifdef NO_MEMMOVE
#define memmove(to, from, length) \
             memcpy((char *) (to), (char *) (from), (int) (length))
#endif

/*
**	$Id: rpm-internal.h,v 1.13 1999/08/09 13:51:33 martin Exp $
*/

extern char* RpmVersion;

/* rpmSupport.c */

/* rpmCmd.c */
int 		rpmBadArgs(Tcl_Interp* interp, int argc, char** argv);
int 		rpmBadNews(Tcl_Interp* interp, char* s);

/* rpmObjStack.c */

void		rpmNewStack();
void		rpmDeleteStack();
int		rpmStackCount();

rpmBinary	rpmNewBinaryLike(rpmBinary r);
rpmBinary	rpmNewBinaryD1(int n);
rpmBinary	rpmNewBinaryD2(int m, int n);
rpmBinary 	rpmDeepCopy(rpmBinary r);
void		rpmPushB(rpmBinary vB);
rpmBinary	rpmPop();
double		rpmPopScalar();
int		rpmStackDepth();

extern rpmBinary rpmBadB;

/* rpmShape.c */
int rpmCatSubCmd(ClientData dummy, Tcl_Interp* interp);
int rpmFlattenSubCmd(ClientData dummy, Tcl_Interp* interp);
int rpmReshapeSubCmd(ClientData dummy, Tcl_Interp* interp);
int rpmTransposeSubCmd(ClientData dummy, Tcl_Interp* interp);
int rpmFlipSubCmd(ClientData dummy, Tcl_Interp* interp);

/* rpmCompare.c */
int rpmEqSubCmd(ClientData dummy, Tcl_Interp* interp);
int rpmGreaterSubCmd(ClientData dummy, Tcl_Interp* interp);
int rpmLesserSubCmd(ClientData dummy, Tcl_Interp* interp);

/* rpmArith.c */
int rpmAddSubCmd(ClientData, Tcl_Interp*);
int rpmSubtractSubCmd(ClientData, Tcl_Interp*);
int rpmMultiplySubCmd(ClientData, Tcl_Interp*);
int rpmDivideSubCmd(ClientData, Tcl_Interp*);
int rpmSqrSubCmd(ClientData, Tcl_Interp*);
int rpmDeqSubCmd(ClientData, Tcl_Interp*);
int rpmNeqSubCmd(ClientData, Tcl_Interp*);
int rpmLtSubCmd(ClientData, Tcl_Interp*);
int rpmLeSubCmd(ClientData, Tcl_Interp*);
int rpmGtSubCmd(ClientData, Tcl_Interp*);
int rpmGeSubCmd(ClientData, Tcl_Interp*);
int rpmCmpSubCmd(ClientData, Tcl_Interp*);
int rpmNotSubCmd(ClientData, Tcl_Interp*);

/* rpmMisc.c */
int rpmRangeSubCmd(ClientData, Tcl_Interp*);
int rpmLengthSubCmd(ClientData, Tcl_Interp*);
int rpmMinSubCmd(ClientData, Tcl_Interp*);
int rpmMaxSubCmd(ClientData, Tcl_Interp*);
int rpmSumSubCmd(ClientData, Tcl_Interp*);
int rpmPsumSubCmd(ClientData, Tcl_Interp*);
int rpmSumsqSubCmd(ClientData, Tcl_Interp*);
int rpmStatsSubCmd(ClientData, Tcl_Interp*);
int rpmSortSubCmd(ClientData, Tcl_Interp*);
int rpmScaletoSubCmd(ClientData, Tcl_Interp*);
int rpmAbscissateSubCmd(ClientData, Tcl_Interp*);
int rpmDecimateSubCmd(ClientData, Tcl_Interp*);
int rpmToplotSubCmd(ClientData, Tcl_Interp*);
int rpmThresholdSubCmd(ClientData, Tcl_Interp*);

/* rpmSmooth.c */
int rpmRunavgSubCmd(ClientData, Tcl_Interp*);
int rpm3rsshSubCmd(ClientData, Tcl_Interp*);

/* rpmGenerate.c */
int rpmSequenceSubCmd(ClientData, Tcl_Interp*);
int rpmUrandomSubCmd(ClientData, Tcl_Interp*);
int rpmReplicateSubCmd(ClientData, Tcl_Interp*);
int rpmExpandSubCmd(ClientData, Tcl_Interp*);
int rpmSExpandSubCmd(ClientData, Tcl_Interp*);
int rpmShrinkSubCmd(ClientData, Tcl_Interp*);

/* rpmApply.c */
int rpmIndexSubCmd(ClientData, Tcl_Interp*);
int rpmSelectSubCmd(ClientData, Tcl_Interp*);

/* rpmDft.c */
int rpmR2cCmd(ClientData, Tcl_Interp*);
int rpmRealCmd(ClientData, Tcl_Interp*);
int rpmImagCmd(ClientData, Tcl_Interp*);
int rpmDftCmd(ClientData, Tcl_Interp*);
int rpmIdftCmd(ClientData, Tcl_Interp*);
int rpmShiftDftCmd(ClientData, Tcl_Interp*);
int rpmTfilterCmd(ClientData, Tcl_Interp*);
int rpmGetrealCmd(ClientData, Tcl_Interp*);

/* rpmElementary.c */
int rpmFloorSubCmd(ClientData, Tcl_Interp*);
int rpmCeilSubCmd(ClientData, Tcl_Interp*);
int rpmAbsSubCmd(ClientData, Tcl_Interp*);
int rpmRintSubCmd(ClientData, Tcl_Interp*);
int rpmExpSubCmd(ClientData, Tcl_Interp*);
int rpmLogSubCmd(ClientData, Tcl_Interp*);
int rpmLog10SubCmd(ClientData, Tcl_Interp*);
int rpmSqrtSubCmd(ClientData, Tcl_Interp*);
int rpmSinSubCmd(ClientData, Tcl_Interp*);
int rpmCosSubCmd(ClientData, Tcl_Interp*);
int rpmTanSubCmd(ClientData, Tcl_Interp*);
int rpmAsinSubCmd(ClientData, Tcl_Interp*);
int rpmAcosSubCmd(ClientData, Tcl_Interp*);
int rpmAtanSubCmd(ClientData, Tcl_Interp*);
int rpmModSubCmd(ClientData, Tcl_Interp*);
int rpmRemainderSubCmd(ClientData, Tcl_Interp*);
int rpmAtan2SubCmd(ClientData, Tcl_Interp*);

/* rpmList.c */
int rpmDimensionsSubCmd(ClientData, Tcl_Interp*);
int rpmGetSubCmd(ClientData, Tcl_Interp*);
int rpmPutSubCmd(ClientData, Tcl_Interp*);

#endif
