#include <stdlib.h>
#include <ctype.h>
#include <tcl.h>
#include <stdio.h>
#include <math.h>
#include <string.h>		/* strcat */

#include "rpm-internal.h"

static char rcsid[] = "$Id: rpmArith.c,v 1.6 1999/08/09 13:51:11 martin Exp $";

static rpmFloat sumop(rpmFloat a, rpmFloat b)
{
    return a + b;
}

static rpmFloat diffop(rpmFloat a, rpmFloat b)
{
    return a - b;
}

static rpmFloat mpyop(rpmFloat a, rpmFloat b)
{
    return a * b;
}

static rpmFloat divop(rpmFloat a, rpmFloat b)
{
    return a / b;
}

static rpmFloat greaterop(rpmFloat a, rpmFloat b)
{
    return (a > b) ? a : b;
}

static rpmFloat lesserop(rpmFloat a, rpmFloat b)
{
    return (a < b) ? a : b;
}

static rpmFloat deqop(rpmFloat a, rpmFloat b)
{
    return a == b;
}

static rpmFloat neqop(rpmFloat a, rpmFloat b)
{
    return a != b;
}

static rpmFloat ltop(rpmFloat a, rpmFloat b)
{
    return a < b;
}

static rpmFloat leop(rpmFloat a, rpmFloat b)
{
    return a <= b;
}

static rpmFloat gtop(rpmFloat a, rpmFloat b)
{
    return a > b;
}

static rpmFloat geop(rpmFloat a, rpmFloat b)
{
    return a >= b;
}

static rpmFloat cmpop(rpmFloat a, rpmFloat b)
{
	if(a == b)
		return 0.0;
	if(a > b)
		return 1.0;
	return -1.0;
}

/*
**	apply op to stack: list1 list2
*/
static int applyOp(Tcl_Interp* interp, rpmFloat (*oper)(rpmFloat, rpmFloat))
{
    rpmBinary a1;
    int n1;
    rpmBinary a2;
    int n2;
    rpmBinary model;
    int nr;
    int ns;
    rpmBinary r;
    int i;
    rpmFloat e;

    a2 = rpmPop();
    n2 = rpmLength(a2);
    a1 = rpmPop();
    n1 = rpmLength(a1);
    if((n1 < 1) || (n2 < 1)) {
		Tcl_AppendResult(interp, "rpm binary op: one operand has length zero",
						 0);
		return TCL_ERROR;
    }
    if(n2 > n1) {
		nr = n2;
		model = a2;
    } else {
		nr = n1;
		model = a1;
    }
    ns = n1 + n2 - nr;
    r = rpmNewBinary(model.dimens, model.dim);

    for(i = 0; i < ns; i++)
		r.v[i]= oper(a1.v[i], a2.v[i]);
    if(nr > n2) {
		e = a2.v[n2 - 1];
		for(; i < nr; i++) 
			r.v[i]= oper(a1.v[i], e);
    } else {
		e = a1.v[n1 - 1];
		for(; i < nr; i++)
			r.v[i] = oper(e, a2.v[i]);
    }
    rpmFreeBinary(a1);
    rpmFreeBinary(a2);
    rpmPushB(r);
    return TCL_OK;
}
/*
**	+ list1 list2
*/
int rpmAddSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return applyOp(interp, sumop);
}

/*
**	- list1 list2
*/
int rpmSubtractSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return applyOp(interp, diffop);
}

/*
**	* list1 list2
*/
int rpmMultiplySubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return applyOp(interp, mpyop);
}

/*
**	/ list1 list2
*/
int rpmDivideSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return applyOp(interp, divop);
}

/*
**	list1 list2 greater
*/
int rpmGreaterSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return applyOp(interp, greaterop);
}

/*
**	list1 list2 lesser
*/
int rpmLesserSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return applyOp(interp, lesserop);
}


/*
**	list sqr
*/
int rpmSqrSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    rpmBinary a;
    int n;
    rpmBinary r;
    int i;

    a = rpmPop();
    r = rpmNewBinary(a.dimens, a.dim);
    n = rpmLength(a);
    for(i = 0; i < n; i++)
		r.v[i] = a.v[i] * a.v[i];
    rpmFreeBinary(a);
    rpmPushB(r);
    return TCL_OK;
}

/*
**	list1 list2 ==
*/
int rpmDeqSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return applyOp(interp, deqop);
}

/*
**	list1 list2 !=
*/
int rpmNeqSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return applyOp(interp, neqop);
}

/*
**	list1 list2 <
*/
int rpmLtSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return applyOp(interp, ltop);
}

/*
**	list1 list2 <=
*/
int rpmLeSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return applyOp(interp, leop);
}

/*
**	list1 list2 >
*/
int rpmGtSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return applyOp(interp, gtop);
}

/*
**	list1 list2 >=
*/
int rpmGeSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return applyOp(interp, geop);
}

/*
**	list1 list2 <=>
*/
int rpmCmpSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    return applyOp(interp, cmpop);
}

/*
**	list !
*/
int rpmNotSubCmd(ClientData dummy, Tcl_Interp* interp)
{
    rpmBinary a;
    int n;
    rpmBinary r;
    int i;

    a = rpmPop();
    r = rpmNewBinary(a.dimens, a.dim);
    n = rpmLength(a);
    for(i = 0; i < n; i++)
		r.v[i] = (a.v[i] != 0.0);
    rpmFreeBinary(a);
    rpmPushB(r);
    return TCL_OK;
}

